#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

#   cambia_capas_cype.py
#
#   Copyright (C) 2007-2008 por Rafael Villar Burke
#   Proyecto Pascaline <pascaline@pascaline.rvburke.com>
#
#   Este programa es software libre. Puede redistribuirlo y/o modificarlo bajo los
#   trminos de la Licencia Pblica General de GNU segn es publicada por la Free
#   Software Foundation, bien de la versin 2 de dicha Licencia o bien (segn su
#   eleccin) de cualquier versin posterior.
#
#   Este programa se distribuye con la esperanza de que sea til, pero SIN NINGUNA
#   GARANTA, incluso sin la garanta MERCANTIL implcita o sin garantizar la
#   CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la copia de la Licencia Pblica
#   General de GNU que debe haber recibido en el archivo COPYING para ms detalles.
#   Puede solicitar una copia de la licencia por escrito a la Free Software
#   Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.

"""Traductor de capas dxf

La aplicacin usa el archivo capas.txt en el directorio del programa para obtener un
diccionario de traduccin de capas."""

import os, sys
import re

TEST = False
TEST_INPUT_DIR = os.path.join(os.getcwd(), '..', 'test', 'brutos')
TEST_OUTPUT_DIR = os.path.join(os.getcwd(), '..', 'test', 'procesados')
TEST_INPUT_FILE = os.path.join(TEST_INPUT_DIR, 'test_input.dxf')
TEST_OUTPUT_FILE = os.path.join(TEST_OUTPUT_DIR, 'test_output.dxf')

DEFAULT_DICCIONARIO_CAPAS_FILE = os.path.join(os.getcwd(), 'capas.txt')
DICCIONARIOS_CAPAS = {}

# Leemos el mapa de capas
def obten_diccionario_capas(archivo=None, separador='|'):
    """Devuelve un diccionario con claves y valores a partir de un archivo.

    El archivo contiene lneas 'Clave|traduccion' en el que las lneas
    en blanco, las que empiezan con #, o las claves sin traduccin se ignoran.
    La funcin devuelve un diccionario de traduccin dict[capa_origen] = capa_destino.
    """
    archivo = os.path.normpath(archivo)
    archivo_key = archivo.lower()
    # hacer cache de archivos:
    if archivo_key in DICCIONARIOS_CAPAS:
        #print "En cach: %s" % archivo_key
        return DICCIONARIOS_CAPAS[archivo_key]
    #print "Fuera de cach: %s" % archivo_key

    _repl_dict_capas = {}
    if archivo is not None:
        try:
            _capas_archivo = open(archivo)
            try:
                for linea in _capas_archivo:
                    if linea.startswith('#') or linea.strip() is "":
                        continue
                    tupla = linea.split(separador)
                    original, destino = tupla[0].strip(), tupla[1].strip()
                    if destino is "":
                        continue
                    _repl_dict_capas[original] = destino
            except:
                raise NameError, 'Archivo mal formado'
        except:
            raise NameError, 'Imposible abrir el archivo de capas'
        _capas_archivo.close()
    DICCIONARIOS_CAPAS[archivo_key] = _repl_dict_capas.copy()
    return DICCIONARIOS_CAPAS[archivo_key]

def _traduce_capas(input=None, output=None, diccionario_traduccion={}):
    """Devuelve la entrada con las claves de un diccionario sustituidas por sus valores.
    """
    def pattern_from_keys(diccionario):
        _keys = diccionario.keys()
        _keys.sort() # lexical order
        _keys.reverse() # use longest match first
        return re.compile("|".join(map(re.escape, _keys)))
    _pattern = pattern_from_keys(diccionario_traduccion)
    def repl(match):
        _item = match.group(0)
        return diccionario_traduccion.get(_item, _item) #dict[item] or item
    for s in input:
        output.write(_pattern.sub(repl, s))

def _abre_streams(input_file=None, output_file=None):
    """Abre archivos de entrada y salida.
    Usa sys.stdin y sys.stdout si no se especifican los nombres de archivo.
    """
    if input_file is None:
        input = sys.stdin
    else:
        try:
            input = open(input_file)
        except:
            raise NameError, 'Error al abrir el archivo de entrada'
    if output_file is None:
        output = sys.stdout
    else:
        try:
            output = open(output_file, 'w')
        except:
            raise NameError, 'Error al abrir el archivo de salida'
    return input, output

def _cierra_streams(input, output):
    """Cierra entrada y salida si son archivos.
    """
    if input is not sys.stdin:
        input.close()
    if output is not sys.stdout:
        output.close()

def procesa_capas_dxf(input_file, output_file, diccionario_capas_file=DEFAULT_DICCIONARIO_CAPAS_FILE):
    """Sustituye las capas del archivo de entrada en el archivo de salida, usando un
    diccionario de traducccin.
    """
    diccionario_capas = obten_diccionario_capas(diccionario_capas_file)
    input, output = _abre_streams(input_file, output_file)
    _traduce_capas(input, output, diccionario_capas)
    _cierra_streams(input, output)

if __name__ == '__main__':
    TEST = True
    print """Traductor de capas dxf\n(c) Rafael Villar Burke, 2007-2008
Este programa se distribuye bajo licencia GPL."""
    print "Procesando archivo..."
    if len(sys.argv) == 1: #sin parmetros
        if TEST:
            input_file = TEST_INPUT_FILE
            output_file = TEST_OUTPUT_FILE
        else:
            print "Cambia capas de un archivo utilizando un diccionario de traduccin"
            print "uso: %s [infile [outfile]]" % os.path.basename(sys.argv[0])
            sys.exit()
    else:
        if len(sys.argv) == 2:
            input_file=sys.argv[1]
            output_file=None
        if len(sys.argv) == 3:
            input_file=sys.argv[1]
            output_file=sys.argv[2]

    import time
    t0 = time.time()
    procesa_capas_dxf(input_file, output_file)
    t1 = time.time()
    print "Procesado finalizado en %.1f segundos" % ((t1 - t0),)
